<?php
/**
 * Login Dongle Plugin
 * 
 * @author Andrea Ercolino, http://notelog.com
 */

class LoginDonglePlugin
{
    const MAXLEN_CHALLENGE = 10;
    const MAXLEN_ANSWER    = 10;
    const MAXLEN_MESSAGE   = 1000;
    
    //do not change below this line
    
    const OPTION_GROUP = 'login_dongle_settings';
    const OPTION_NAME  = 'login_dongle';                //used in the db
    
    const PAGE_ID = 'login_dongle_configuration';
    const CORE_ID = 'core_settings';
    const NICE_ID = 'nice_settings';
    
    const CHALLENGE_PREFIX = 'logindongle_';            //to prevent field clashes
    
    public function __construct()
    {
        add_action('wp_loaded', array($this, 'validateLogin'), 1);
        
        add_action('admin_menu', array($this, 'adminMenu'));
        add_action('admin_init', array($this, 'adminInit'));
    }
    
    public function validateLogin()
    {
        if (ABSPATH . 'wp-login.php' != $_SERVER['SCRIPT_FILENAME'])
        {
            return; //if not the login page
        }
        if ('POST' != $_SERVER['REQUEST_METHOD'])
    	{
    		return; //if not a POST
    	}
    	$options = get_option(self::OPTION_NAME);
    	if ('' == $options['challenge'])
    	{
    	    return; //if a challenge is not defined into settings
    	}
    	//wp uses magic quotes always, see http://lists.automattic.com/pipermail/wp-hackers/2010-May/031765.html
    	$challenge = addslashes(self::CHALLENGE_PREFIX . $options['challenge']);
    	$response  = addslashes($options['response']);
    	$isValid = isset($_POST[$challenge]) && $_POST[$challenge] == $response;
    	if (! $isValid) 
    	{ 
    		wp_die($options['message']);
    	}
    	unset($_POST[$challenge], $_REQUEST[$challenge]);
    }
    
    protected function bookmarklet()
    {
        $options   = get_option(self::OPTION_NAME);
    	$challenge = $options['challenge'];
    	if ('' == $challenge)
    	{
    	    return;
    	}
    	$blog_url = preg_replace('@^(https|http)://@', '', home_url());
    	//the following JS is semi-minified, just remove double spaces and new lines to fully minify it
    	$result = <<< END_source
(function($){
  var action=jQuery('#loginform').attr('action')||'';
  if(action.search(/^(https|http):\/\/BLOG_URL\\b/i)==-1)
    return alert('No login form or not targeting your blog.');
  var challenge='CHALLENGE';
  var response=prompt(challenge,'');
  $('<input type="hidden">')
    .appendTo('#loginform')
    .attr('name','PREFIX'+challenge)
    .attr('value',response);
  $('#wp-submit')
    .click()
})(jQuery);
END_source;
        $result = str_replace('  ', '', $result);
        $result = preg_replace("@\r?\n@", '', $result);
        $result = preg_replace('@\bBLOG_URL\b@', preg_quote($blog_url, '/'), $result);
    	$result = preg_replace('@\bPREFIX\b@', self::CHALLENGE_PREFIX, $result);
    	$result = preg_replace('@\bCHALLENGE\b@', addslashes($challenge), $result);
    	return $result;
    }
    
    public function adminMenu()
    {
        add_options_page('Login Dongle Settings', 'Login Dongle', 'manage_options', self::PAGE_ID, array($this, 'adminPage'));
    }
    
    public function adminPage()
    {
        $code = $this->bookmarklet();
        $dongle_pc = 'javascript:' . htmlspecialchars(rawurlencode($code), ENT_COMPAT); //safest
        $dongle_sp = 'javascript:' . htmlspecialchars($code, ENT_COMPAT);
?>
<div>
    <h2>Login Dongle Settings</h2>
    <p>Login Dongle makes the login <strong>for your personal use only</strong>, by means of a 
    conventional challenge &gt;&gt; response mechanism.</p>
    <form action="options.php" method="post">
<?php 
        settings_fields(self::OPTION_GROUP); 
        do_settings_sections(self::PAGE_ID);
?>
        <p><br />
        <input type="submit" name="Submit" value="<?php esc_attr_e('Save Changes'); ?>" 
        class="button" /></p>
    </form>
    <p>After saving any changes, you should <strong>drag and drop into your bookmarks this <a 
    href="<?php echo $dongle_pc; ?>" title="Log In">login dongle</a> bookmarklet</strong>.<br />
    To be able in the future to log into your blog using a different browser or a different 
    system, backup its code to a private web location (like a message into your web mail 
    account).<br/>
    Please note that you yourself won't be able to log into your blog without your most current 
    login dongle, unless you disable this plugin right from the file system, as described in the
    <a href="http://wordpress.org/extend/plugins/login-dongle/faq/" target="_blank">FAQ</a>.</p>
    <table class="form-table">
    	<tbody>
    		<tr valign="top">
    			<th scope="row">Encoded, for your PC</th>
    			<td>
    				<textarea style="width:60%;"><?php echo $dongle_pc; ?></textarea>
				</td>
			</tr>
    		<tr valign="top">
    			<th scope="row">Not encoded, for your SmartPhone</th>
    			<td>
    				<textarea style="width:60%;"><?php echo $dongle_sp; ?></textarea>
				</td>
			</tr>
		</tbody>
	</table>
</div>
<?php
    }
    
    public function adminValidate( $data )
    {
        $data['challenge'] = substr(trim($data['challenge']), 0, self::MAXLEN_CHALLENGE);
        if (empty($data['challenge']))
        {
            $data['challenge'] = 'challenge';
        }
        $data['response'] = substr(trim($data['response']), 0, self::MAXLEN_ANSWER);
        if (empty($data['response']))
        {
            $data['response'] = 'response';
        }
        $data['message'] = strip_tags($data['message'], '<p><br><a><strong><em>');
        $data['message'] = substr(trim($data['message']), 0, self::MAXLEN_MESSAGE);
        if (empty($data['message']))
        {
            $data['message'] = 'No way in.';
        }
        return $data;
    }
    
    public function adminCoreHelp()
    {
?>
<p>The challenge and its expected response resemble passwords, but are fundamentally different. 
These data is supposed to have enough complexity to deter the casual user of your PC, if any. So 
feel free to use simple pairs, like doit &gt;&gt; now, better if they are meaningful only to you, 
like age &gt;&gt; 95, slightly better if you can associate the response to the challenge but others 
will have a harder time, like color &gt;&gt; ful. (For your security, please do not use these 
examples literally!).</p>
<?php
    }
    
    public function adminNiceHelp()
    {
?>
<p>Here you can configure the message displayed to possible attackers.</p>
<?php
    }
    
    function adminChallengeField() 
    {
        $options = get_option(self::OPTION_NAME);
?>
<input type="text" id="challenge" 
	name="<?php echo self::OPTION_NAME; ?>[challenge]" value="<?php echo htmlspecialchars($options['challenge']); ?>"
	size="<?php echo (self::MAXLEN_CHALLENGE + 2); ?>" maxlength="<?php echo self::MAXLEN_CHALLENGE; ?>"  />
<?php
    }
    
    function adminResponseField() 
    {
        $options = get_option(self::OPTION_NAME);
?>
<input type="text" id="response" 
	name="<?php echo self::OPTION_NAME; ?>[response]" value="<?php echo htmlspecialchars($options['response']); ?>"
	size="<?php echo (self::MAXLEN_ANSWER + 2); ?>" maxlength="<?php echo self::MAXLEN_ANSWER; ?>"  />
<?php
    }
    
    function adminMessageField() 
    {
        $options = get_option(self::OPTION_NAME);
?>
<textarea id="message" name="<?php echo self::OPTION_NAME; ?>[message]" ><?php 
    echo $options['message'];
    ?></textarea>
<?php
    }
    
    public function adminInit()
    {
        register_setting(self::OPTION_GROUP, self::OPTION_NAME, array($this, 'adminValidate'));
        
        add_settings_section(self::CORE_ID, 'Core Settings', array($this, 'adminCoreHelp'), self::PAGE_ID);
        add_settings_field('challenge', 'Challenge', array($this, 'adminChallengeField'), self::PAGE_ID, self::CORE_ID);
        add_settings_field('response', 'Response', array($this, 'adminResponseField'), self::PAGE_ID, self::CORE_ID);
        
        add_settings_section(self::NICE_ID, 'Nice Settings', array($this, 'adminNiceHelp'), self::PAGE_ID);
        add_settings_field('message', 'Message', array($this, 'adminMessageField'), self::PAGE_ID, self::NICE_ID);
    }
}
